(** PCF simple

   Il y a trois sous-modules {!Ast} (dfinition des termes
   de PCF), {!Reader} (dfinition des lecteurs de termes),
   et {!Top} (pour fabriquer une boucle interactive, du genre
   de celle de {b ocaml} *)

(** Types des arbres de syntaxe abstraite *)
module Ast : sig

type t =
  | Num of int              (** Constante entire *)
  | Var of string           (** Accs  une variable *)
  | Op  of Op.t * t * t     (** Application d'un oprateur binaire [t1 op t2] *)
  | Ifz of t * t * t        (** Conditionnelle [Ifz t1 Then t2 Else t3] *)
  | Let of string * t * t   (** Liaison locale [Let x = t1 In t2] *) 
  | App of t * t            (** Application de fonction [t1 t2]*)
  | Fun of string * t       (** Dfinition de fonction [Fun x -> t] *)
  | Fix of string * t       (** Point fixe [Fix x -> t] *)

val print : out_channel -> t -> unit
(** Afficher dans un canal de sortie.
    Un canal simple est la sortie standard.
     @see <http://caml.inria.fr/pub/docs/manual-ocaml/libref/Pervasives.html#VALstdout> [Pervasives.stdout] *)


end

(** Lecture des termes PCF *)
module Reader : sig

  val from_chan : in_channel -> Ast.t
(** Lire dans un canal (un fichier), un canal simple est l'entre standard
    [stdin].
    @see <http://caml.inria.fr/pub/docs/manual-ocaml/libref/Pervasives.html#VALstdin> [Pervasives.stdin] *)

  val from_string : string -> Ast.t
(** Lire dans une chane *)

end

(** Boucle interactive  pour PCF *)
module Top : sig

val loop :
     (out_channel -> 'a -> unit) ->
     ('a Env.t -> Ast.t -> 'a) ->
     unit
(** [loop print eval] lance une boucle 'toplevel', en franais boucle interactive.
    Une telle boucle permet d'valuer une srie de termes PCF dans
    un environnement qui va croissant.

    Une fois lance la boucle (par [loop print eval]), on voit une invite
    du style {b PCF> }. En rponse  cette invite on peut crire,
      - Un terme PCF (suivi d'un double point virgule) [t;;].
      - Ou une dfinition ordinaire [Let x = t;;].
      - Ou une dfinition rcursive [Let Rec x = t;;],
        (qui est lue comme [Let x = Fix x -> t;;], ce qui nous ramne au
         cas prcdent).

    Dans tous les cas, la boucle appelle la fonction [eval] fournie
    comme [eval env t], o
      - le terme PCF [t] est donn ci-dessus,
      - [env] est un environnement initialement vide.

    Cet appel renvoie un rsulat [r] de type ['a], qui est
    affich par la fonction [print] fournie.

    Enfin, la boucle interactive se rappelle, mais avec un
    environnement tendu par la liaison de [x]  [r] dans le
    cas o l'on avait crit [Let x = t;;].

   L'effet gnral est celui de la boucle interactive de Caml.
   @see <../index.html#top> Exemple d'utilisation de [S.Top.loop] (source {b intertop.ml}).
*)

end

